@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2017  Ruby-GNOME2 Project Team
+# Copyright (C) 2016-2018  Ruby-GNOME2 Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -62,7 +62,7 @@ module GdkPixbuf
       when 5
         message = "#{caller[0]}: "
         message << "#{self.class}.new(colorspace, has_alpha, bits_per_sample, "
-        message << "width, height) is deprecated."
+        message << "width, height) is deprecated. "
         message << "Use #{self.class}.new(:colorspace => colorspace, "
         message << ":has_alpha => has_alpha, "
         message << ":bits_per_sample => bits_per_sample, "
@@ -73,7 +73,7 @@ module GdkPixbuf
       when 7
         message = "#{caller[0]}: "
         message << "#{self.class}.new(data, colorspace, has_alpha, "
-        message << "bits_per_sample, width, height) is deprecated."
+        message << "bits_per_sample, width, height) is deprecated. "
         message << "Use #{self.class}.new(:data => data, "
         message << ":colorspace => colorspace, "
         message << ":has_alpha => has_alpha, "
@@ -92,17 +92,17 @@ module GdkPixbuf
       has_alpha = options[:has_alpha]
       has_alpha = false if has_alpha.nil?
       bits_per_sample = options[:bits_per_sample] || 8
-      row_stride = options[:row_stride] || 0
-      data = options[:data] || nil
-      bytes = options[:bytes] || nil
-      xpm = options[:xpm] || nil
-      file = options[:file] || nil
-      # stream = options[:stream] || nil
-      resource = options[:resource] || nil
-      width = options[:width] || nil
-      height = options[:height] || nil
+      row_stride = options[:row_stride]
+      data = options[:data]
+      bytes = options[:bytes]
+      xpm = options[:xpm]
+      file = options[:file]
+      # stream = options[:stream]
+      resource = options[:resource]
+      width = options[:width]
+      height = options[:height]
       size = true if width && height
-      scale = options[:scale] || nil
+      scale = options[:scale]
       preserve_aspect_ratio = options[:preserve_aspect_ratio]
       preserve_aspect_ratio = true if preserve_aspect_ratio.nil?
 
@@ -119,12 +119,26 @@ module GdkPixbuf
       elsif resource && !scale
         initialize_new_from_resource(resource)
       elsif data && size
-        initialize_new_from_data(data, colorspace, has_alpha, bits_per_sample,
-                                 width, height, row_stride)
+        data = data.pack("C*") if data.is_a?(Array)
+        row_stride ||= data.bytesize / height
+        initialize_new_from_data(data,
+                                 colorspace,
+                                 has_alpha,
+                                 bits_per_sample,
+                                 width,
+                                 height,
+                                 row_stride)
+        @data = data
       elsif bytes && size
-         initialize_new_from_bytes(bytes, colorspace, has_alpha, bits_per_sample,
-                                   width, height, row_stride)
-
+        row_stride ||= bytes.size / height
+        initialize_new_from_bytes(bytes,
+                                  colorspace,
+                                  has_alpha,
+                                  bits_per_sample,
+                                  width,
+                                  height,
+                                  row_stride)
+        @bytes = bytes
       elsif xpm
         initialize_new_from_xpm_data(xpm)
       elsif size
@@ -157,18 +171,7 @@ module GdkPixbuf
     end
     private :initialize_with_hash
 
-    if private_method_defined?(:initialize_new_from_file_utf8)
-      alias_method :initialize_new_from_file,
-                   :initialize_new_from_file_utf8
-    end
-    if private_method_defined?(:initialize_new_from_file_at_size_utf8)
-      alias_method :initialize_new_from_file_at_size,
-                   :initialize_new_from_file_at_size_utf8
-    end
-    if private_method_defined?(:initialize_new_from_file_at_scale_utf8)
-      alias_method :initialize_new_from_file_at_scale,
-                   :initialize_new_from_file_at_scale_utf8
-    end
+    alias_method :row_stride, :rowstride
 
     def dup
       copy
@@ -223,8 +226,22 @@ module GdkPixbuf
     def save(*args)
       case args.size
       when 1
-        filename = nil
-        type, = args
+        arg1 = args[0]
+        if arg1.respond_to?(:to_path)
+          filename = arg1.to_path
+        elsif arg1.is_a?(String) and arg1.include?(".")
+          filename = args[0]
+        else
+          filename = nil
+          type = arg1
+        end
+        if filename
+          type = File.extname(filename).gsub(/\A\./, "").downcase
+          case type
+          when "jpg"
+            type = "jpeg"
+          end
+        end
         options = {}
       when 2
         if args.last.is_a?(Hash)
@@ -292,11 +309,11 @@ module GdkPixbuf
       dest_width = options[:destination_width] || options[:dest_width]
       dest_height = options[:destination_height] || options[:dest_height]
 
-      destination ||= Pixbuf.new(colorspace,
-                                 has_alpha?,
-                                 bits_per_sample,
-                                 dest_x + dest_width,
-                                 dest_y + dest_height)
+      destination ||= Pixbuf.new(:colorspace => colorspace,
+                                 :has_alpha => has_alpha?,
+                                 :bits_per_sample => bits_per_sample,
+                                 :width => dest_x + dest_width,
+                                 :height => dest_y + dest_height)
       destination.composite!(self, options)
       destination
     end
